<?php

namespace Drupal\svg_image_graphql\Plugin\GraphQL\Fields\Entity\Fields\Image;

use Drupal\graphql\GraphQL\Execution\ResolveContext;
use Drupal\graphql\Plugin\GraphQL\Fields\FieldPluginBase;
use Drupal\image\Plugin\Field\FieldType\ImageItem;
use GraphQL\Type\Definition\ResolveInfo;

/**
 * Retrieve if the image is a SVG.
 *
 * @GraphQLField(
 *   id = "image_svg",
 *   secure = true,
 *   name = "svg",
 *   type = "String",
 *   field_types = {"image"},
 *   provider = "image",
 *   deriver = "Drupal\graphql_core\Plugin\Deriver\Fields\EntityFieldPropertyDeriver"
 * )
 */
class Svg extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  protected function resolveValues($value, array $args, ResolveContext $context, ResolveInfo $info) {
    if ($value instanceof ImageItem && $value->entity && $value->entity->access('view') && svg_image_is_file_svg($value->entity)) {
      $fileUri = $value->entity->getFileUri();

      if (file_exists($fileUri)) {
        $svgRaw = file_get_contents($fileUri);
        $svgRaw = preg_replace(['/<\?xml.*\?>/i', '/<!DOCTYPE((.|\n|\r)*?)">/i'], '', $svgRaw);
        $svgRaw = trim($svgRaw);
      }
      else {
        $svgRaw = NULL;
      }
      yield $svgRaw;
    }
  }

}
