<?php

namespace Drupal\svg_image_graphql\Plugin\GraphQL\Fields\Entity\Fields\Image;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\file\Entity\File;
use Drupal\graphql\GraphQL\Cache\CacheableValue;
use Drupal\graphql\GraphQL\Execution\ResolveContext;
use Drupal\graphql_core\Plugin\GraphQL\Fields\Entity\Fields\Image\ImageDerivative;
use Drupal\image\Entity\ImageStyle;
use Drupal\image\Plugin\Field\FieldType\ImageItem;
use GraphQL\Type\Definition\ResolveInfo;

/**
 * Retrieve the image field derivative (image style).
 *
 * @GraphQLField(
 *   id = "svg_image_derivative",
 *   secure = true,
 *   name = "svg_derivative",
 *   type = "ImageResource",
 *   arguments = {
 *     "style" = "ImageStyleId!"
 *   },
 *   field_types = {"image"},
 *   provider = "image",
 *   deriver = "Drupal\graphql_core\Plugin\Deriver\Fields\EntityFieldPropertyDeriver"
 * )
 */
class SvgImageDerivative extends ImageDerivative implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  protected function resolveValues($value, array $args, ResolveContext $context, ResolveInfo $info) {
    if ($value instanceof ImageItem && $value->entity && $value->entity->access('view') && $style = ImageStyle::load($args['style'])) {
      assert($style instanceof ImageStyle);
      /** @var \Drupal\file\Entity\File $file */
      $file = $value->entity;
      assert($file instanceof File);

      // Determine the dimensions of the styled image.
      $dimensions = [
        'width' => $value->width,
        'height' => $value->height,
      ];

      if ($style->supportsUri($file->getFileUri())) {
        $style->transformDimensions($dimensions, $file->getFileUri());
        $url = $style->buildUrl($file->getFileUri());
      }
      else {
        $url = $file->createFileUrl(FALSE);
      }

      // If this is an SVG image, or it is a GIF,
      // we return always the url to the original file.
      if (svg_image_is_file_svg($file) || $file->getMimeType() === 'image/gif') {
        $url = $file->createFileUrl(FALSE);
      }

      yield new CacheableValue([
        'url' => $url,
        'width' => $dimensions['width'],
        'height' => $dimensions['height'],
      ], [$style]);
    }
  }

}
